/******************************************
 * xatm_cap.c
 *    This file comes from Xatm's Xatm-Cap
 *
 *          by:  ͵ <yokota@netlab.istsukuba.ac.jp>
 *
 * This file uses GNU's extended termcap lib.
 * See GNU termcap info file.
 */
/* $Id: xatm_cap.c,v 1.2 1999/09/29 20:04:04 elca Exp elca $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <font.h>

#include "fntfilst.h"
#include "t1intf.h"
#include "range.h"
#include "xatm_cap.h"

#include "../../../termcap-1.3-xatm/termcap.h"

/***************************************************************
 *
 * "WadaGo-Regular--H@italic.cid" -> "WadaGo-Regular"
 * "WadaGo-Regular--H.cid"        -> "WadaGo-Regular"
 * "WadaGo-Regular@italic.cid"    -> "WadaGo-Regular"
 * "WadaGo-Regular.cid"           -> "WadaGo-Regular"
 */
int cid_get_font_name(const char *filename, char *fontname)
{
	char buf[CID_NAME_MAX];
	char *dash;
	char *at_mark;
	char *ext_dot;

	strncpy(buf, filename, CID_NAME_MAX);

	dash = strstr(buf, "--");
	if(dash != NULL) {
		*dash = '\0';
		goto success_return;
	}

	at_mark = strchr(buf, '@');
	if(at_mark != NULL) {
		*at_mark = '\0';
		goto success_return;
	}

	ext_dot = strrchr(buf, '.');
	if(ext_dot != NULL) {
		*ext_dot = '\0';
		goto success_return;
	}

	return BadFontName;

success_return:
	strncpy(fontname, buf, CID_NAME_MAX);
	return Successful;
}

/***************************************************************
 *
 * "WadaGo-Regular--H@italic.cid" -> "H"
 * "WadaGo-Regular--H.cid"        -> "H"
 * "WadaGo-Regular@italic.cid"    -> error
 * "WadaGo-Regular.cid"           -> error
 */
int cid_get_cmap_name(const char *filename, char *cmapname)
{
	char buf[CID_NAME_MAX];
	char *dash;
	char *at_mark;
	char *ext_dot;
	char *name_start;
	int len;

	strncpy(buf, filename, CID_NAME_MAX);

	dash = strstr(buf, "--");
	if(dash == NULL) {
		goto error_exit;
	}
	name_start = dash + 2;

	at_mark = strchr(buf, '@');
	ext_dot = strrchr(buf, '.');
	if(at_mark != NULL) {
		len = at_mark - name_start;
	} else if(ext_dot != NULL) {
		len = ext_dot - name_start;
	} else {
		goto error_exit;
	}

	strncpy(cmapname, name_start, len);
	cmapname[len] = '\0';

	return Successful;

error_exit:
	cmapname[0] = '\0';
	return BadFontName;
}

/***************************************************************
 *
 * "WadaGo-Regular--H@italic.cid" -> "italic"
 * "WadaGo-Regular--H.cid"        -> error
 * "WadaGo-Regular@italic.cid"    -> "italic"
 * "WadaGo-Regular.cid"           -> error
 */
int cid_get_cap_name(const char *filename, char *capname)
{
	char buf[CID_NAME_MAX];
	char *at_mark;
	char *ext_dot;
	int len;

	strncpy(buf, filename, CID_NAME_MAX);

	at_mark = strrchr(buf, '@');
	if(at_mark == NULL) {
		goto error_exit;
	}

	ext_dot = strrchr(buf, '.');
	if(ext_dot == NULL) {
		goto error_exit;
	}

	if (at_mark > ext_dot) {
		goto error_exit;
	}

	len = ext_dot - at_mark - 1;

	strncpy(capname, at_mark + 1, len);
	capname[len] = '\0';

	return Successful;

error_exit:
	capname[0] = '\0';
	return BadFontName;
}


/*
 */
int cid_caps_change_vals(const char *capname, FontScalablePtr vals)
{
	/* եȤ꤬ʤ BadFontName ֤
	   ξϥ顼ˤΤǤϤʤ
	   Ĥʤäݤϥǥեͤ
	   Ȥ褦ˤ롣 */
	if(tgetent(NULL, capname) < 1) {
		return BadFontName;
	}

	/* italic */
        if(tgetflag("it")) {
		vals->pixel_matrix[2] += vals->pixel_matrix[3] * 0.25;
		vals->point_matrix[2] += vals->point_matrix[3] * 0.25;
	}

	return Successful;
}
